"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
require("mocha");
var _chai = _interopRequireWildcard(require("chai"));
var _sinon = _interopRequireDefault(require("sinon"));
var _sinonChai = _interopRequireDefault(require("sinon-chai"));
var _moduleRegistry = _interopRequireDefault(require("../src/module-registry"));
var _ReactModuleContainerErrors = require("../src/ReactModuleContainerErrors");
function _getRequireWildcardCache(e) { if ("function" != typeof WeakMap) return null; var r = new WeakMap(), t = new WeakMap(); return (_getRequireWildcardCache = function _getRequireWildcardCache(e) { return e ? t : r; })(e); }
function _interopRequireWildcard(e, r) { if (!r && e && e.__esModule) return e; if (null === e || "object" != typeof e && "function" != typeof e) return { "default": e }; var t = _getRequireWildcardCache(r); if (t && t.has(e)) return t.get(e); var n = { __proto__: null }, a = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var u in e) if ("default" !== u && Object.prototype.hasOwnProperty.call(e, u)) { var i = a ? Object.getOwnPropertyDescriptor(e, u) : null; i && (i.get || i.set) ? Object.defineProperty(n, u, i) : n[u] = e[u]; } return n["default"] = e, t && t.set(e, n), n; }
_chai["default"].use(_sinonChai["default"]);
describe('Module Registry', function () {
  beforeEach(function () {
    _moduleRegistry["default"].cleanAll();
  });
  it('should be able to register a module', function () {
    var MyModule = function MyModule() {};
    _moduleRegistry["default"].registerModule('GLOBAL_ID', MyModule);
    var result = _moduleRegistry["default"].getModule('GLOBAL_ID');
    (0, _chai.expect)(result).to.be.an.instanceOf(MyModule);
  });
  it('should be able to pass parameters to the register a module', function () {
    var MyModule = function MyModule(name) {
      this.name = name;
    };
    _moduleRegistry["default"].registerModule('GLOBAL_ID', MyModule, ['DUMMY_NAME']);
    var result = _moduleRegistry["default"].getModule('GLOBAL_ID');
    (0, _chai.expect)(result.name).to.eq('DUMMY_NAME');
  });
  it('should be able to get all modules', function () {
    var MyModule1 = function MyModule1() {};
    var MyModule2 = function MyModule2() {};
    var MyModule3 = function MyModule3() {};
    _moduleRegistry["default"].registerModule('GLOBAL_ID1', MyModule1);
    _moduleRegistry["default"].registerModule('GLOBAL_ID2', MyModule2);
    _moduleRegistry["default"].registerModule('GLOBAL_ID3', MyModule3);
    var allModules = _moduleRegistry["default"].getAllModules();
    (0, _chai.expect)(allModules.length).to.eq(3);
    (0, _chai.expect)(allModules.find(function (m) {
      return m instanceof MyModule1;
    })).to.be.an.instanceOf(MyModule1);
    (0, _chai.expect)(allModules.find(function (m) {
      return m instanceof MyModule2;
    })).to.be.an.instanceOf(MyModule2);
    (0, _chai.expect)(allModules.find(function (m) {
      return m instanceof MyModule3;
    })).to.be.an.instanceOf(MyModule3);
  });
  it('should be able to register a method and call it', function () {
    var method = _sinon["default"].spy();
    _moduleRegistry["default"].registerMethod('GLOBAL_ID', function () {
      return method;
    });
    _moduleRegistry["default"].invoke('GLOBAL_ID', 1, 2, 3);
    (0, _chai.expect)(method).calledWith(1, 2, 3);
  });
  it('should be able to register a component', function () {
    var component = function component() {
      return '<div>FAKE_COMPONENT</div>';
    };
    _moduleRegistry["default"].registerComponent('GLOBAL_ID', component);
    var resultComponent = _moduleRegistry["default"].component('GLOBAL_ID');
    (0, _chai.expect)(resultComponent).to.eq('<div>FAKE_COMPONENT</div>');
  });
  it('should notify all event listeners', function () {
    var listener1 = _sinon["default"].spy();
    var listener2 = _sinon["default"].spy();
    _moduleRegistry["default"].addListener('GLOBAL_ID', listener1);
    _moduleRegistry["default"].addListener('GLOBAL_ID', listener2);
    _moduleRegistry["default"].notifyListeners('GLOBAL_ID', 1, 2, 3);
    (0, _chai.expect)(listener1).calledWith(1, 2, 3);
    (0, _chai.expect)(listener2).calledWith(1, 2, 3);
  });
  it('should clean all the methods, components, events, and modules when calling cleanAll', function () {
    _moduleRegistry["default"].registerModule('GLOBAL_ID', function MyModule() {});
    _moduleRegistry["default"].registerMethod('GLOBAL_ID', function () {
      return function () {};
    });
    _moduleRegistry["default"].registerComponent('GLOBAL_ID', function () {});
    _moduleRegistry["default"].addListener('GLOBAL_ID', function () {});
    _moduleRegistry["default"].cleanAll();
    (0, _chai.expect)(_moduleRegistry["default"].getModule('GLOBAL_ID')).to.be.undefined;
    (0, _chai.expect)(_moduleRegistry["default"].notifyListeners('GLOBAL_ID')).to.be.undefined;
    (0, _chai.expect)(_moduleRegistry["default"].component('GLOBAL_ID')).to.be.undefined;
    (0, _chai.expect)(_moduleRegistry["default"].invoke('GLOBAL_ID')).to.be.undefined;
  });
  describe('ReactModuleContainerError', function () {
    var reactModuleContainerErrorCallback;
    beforeEach(function () {
      reactModuleContainerErrorCallback = _sinon["default"].stub();
      _moduleRegistry["default"].addListener('reactModuleContainer.error', reactModuleContainerErrorCallback);
    });
    it('should be fired when trying to invoke an unregistered method', function () {
      var unregisteredMethodName = 'unregistered-method';
      var result = _moduleRegistry["default"].invoke(unregisteredMethodName);
      (0, _chai.expect)(reactModuleContainerErrorCallback).calledOnce;
      var errorCallbackArg = reactModuleContainerErrorCallback.getCall(0).args[0];
      (0, _chai.expect)(errorCallbackArg).to.be.an["instanceof"](_ReactModuleContainerErrors.UnregisteredMethodInvokedError);
      (0, _chai.expect)(errorCallbackArg.message).to.eq("ModuleRegistry.invoke " + unregisteredMethodName + " used but not yet registered");
      (0, _chai.expect)(result).to.eq(undefined);
    });
    it('should be fired when trying to use an unregistered component', function () {
      var componentId = 'component-id';
      var resultComponent = _moduleRegistry["default"].component(componentId);
      (0, _chai.expect)(reactModuleContainerErrorCallback).calledOnce;
      var errorCallbackArg = reactModuleContainerErrorCallback.getCall(0).args[0];
      (0, _chai.expect)(errorCallbackArg).to.be.an["instanceof"](_ReactModuleContainerErrors.UnregisteredComponentUsedError);
      (0, _chai.expect)(errorCallbackArg.message).to.eq("ModuleRegistry.component " + componentId + " used but not yet registered");
      (0, _chai.expect)(resultComponent).to.eq(undefined);
    });
    it('should be fired when a listener callback throws an error', function () {
      var someRegisteredMethod = 'someRegisteredMethod';
      var error = new Error();
      _moduleRegistry["default"].addListener(someRegisteredMethod, function () {
        throw error;
      });
      _moduleRegistry["default"].notifyListeners(someRegisteredMethod);
      var errorCallbackArg = reactModuleContainerErrorCallback.getCall(0).args[0];
      (0, _chai.expect)(errorCallbackArg).to.be.an["instanceof"](_ReactModuleContainerErrors.ListenerCallbackError);
      (0, _chai.expect)(errorCallbackArg.message).to.eq("Error in listener callback of module registry method: " + someRegisteredMethod);
    });
    it('should report an error if the given module was already registered', function () {
      var MyModule = function MyModule() {};
      var TheirModule = function TheirModule() {};
      ;
      _moduleRegistry["default"].registerModule('GLOBAL_ID', MyModule);
      _moduleRegistry["default"].registerModule('GLOBAL_ID', TheirModule);
      (0, _chai.expect)(reactModuleContainerErrorCallback).calledOnce;
      var errorCallbackArg = reactModuleContainerErrorCallback.getCall(0).args[0];
      (0, _chai.expect)(errorCallbackArg).to.be.an["instanceof"](_ReactModuleContainerErrors.ModuleAlreadyRegisteredError);
      (0, _chai.expect)(errorCallbackArg.message).to.eq("A module with id \"GLOBAL_ID\" is already registered");
      (0, _chai.expect)(_moduleRegistry["default"].getModule('GLOBAL_ID')).to.be.an.instanceOf(MyModule);
    });
  });
});